import simplejson as json
from typing import Any, Callable, Optional
from notbank_python_sdk.core.ap_data_handler import ApDataHandler
from notbank_python_sdk.core.endpoints import Endpoints
from notbank_python_sdk.models.authenticate_response import AuthenticateResponse
from notbank_python_sdk.models.pong import Pong
from notbank_python_sdk.parsing import parse_response_fn
from notbank_python_sdk.requests_models.authenticate_request import AuthenticateRequest

from notbank_python_sdk.core.authenticator import Authenticator
from notbank_python_sdk.client_connection import T
from notbank_python_sdk.core.converter import from_dict, to_dict
from notbank_python_sdk.core.endpoint_category import EndpointCategory
from notbank_python_sdk.error import NotbankException, ErrorCode
from notbank_python_sdk.core.response_handler import ResponseHandler
from notbank_python_sdk.websocket.callback_manager import CallbackManager
from notbank_python_sdk.websocket.handler import WebsocketHandler
from notbank_python_sdk.websocket.subscription import Subscription, Unsubscription
from notbank_python_sdk.websocket.websocket_manager import WebsocketManager
from notbank_python_sdk.websocket.websocket_requester import WebsocketRequester
from notbank_python_sdk.websocket.websocket_response_handler import WebsocketResponseHandler


class WebsocketConnection:
    _callback_manager: CallbackManager
    _websocket_manager: WebsocketManager
    _websocket_requester: WebsocketRequester
    _websocket_response_hanlder: WebsocketResponseHandler

    def __init__(
            self,
            callback_manager: CallbackManager,
            websocket_manager: WebsocketManager,
            websocket_requester: WebsocketRequester,
            websocket_response_handler: WebsocketResponseHandler):
        self._callback_manager = callback_manager
        self._websocket_manager = websocket_manager
        self._websocket_requester = websocket_requester
        self._websocket_response_hanlder = websocket_response_handler

    @staticmethod
    def create(uri: str,
               on_open: Callable[[], None] = lambda: None,
               on_close: Callable[[Any, str], None] = lambda code, message: None,
               on_failure: Callable[[Exception], None] = lambda e: None,
               peek_message_in: Callable[[str], None] = lambda x: None,
               peek_message_out: Callable[[str], None] = lambda x: None,
               request_timeout: Optional[float] = None,
               ) -> 'WebsocketConnection':
        callback_manager = CallbackManager.create()
        response_handler = WebsocketResponseHandler.create(
            callback_manager,
            on_failure)
        websocket_manager = WebsocketManager(
            WebsocketHandler(
                response_handler.handle,
                on_open,
                on_close,
                on_failure),
            uri,
            peek_message_in,
            peek_message_out)
        websocket_requester = WebsocketRequester.create(
            callback_manager,
            websocket_manager.send,
            on_failure,
            request_timeout=request_timeout,
        )
        websocket_response_handler = WebsocketResponseHandler.create(
            callback_manager,
            on_failure)
        return WebsocketConnection(
            callback_manager,
            websocket_manager,
            websocket_requester,
            websocket_response_handler)

    def connect(self) -> None:
        self._websocket_manager.connect()

    def close(self) -> None:
        self._websocket_manager.close()

    def subscribe(self, subscription: Subscription[T]) -> T:
        subscription_result_getter = self._websocket_requester.subscribe(
            subscription)
        result = subscription_result_getter()
        return self.handle_result(subscription.parse_response_fn, result)

    def unsubscribe(self, unsubscription: Unsubscription[T]) -> T:
        unsubscription_result_getter = self._websocket_requester.unsubscribe(
            unsubscription)
        result = unsubscription_result_getter()
        return self.handle_result(unsubscription.parse_response_fn, result)

    def request(
            self,
            endpoint: str,
            endpoint_category: EndpointCategory,
            request_message: Any,
            parse_response_fn: Callable[[Any], T]) -> T:
        if endpoint_category != EndpointCategory.AP:
            raise NotbankException(
                ErrorCode.INVALID_REQUEST, "websocket server only supports ap endpoints")
        result_getter = self._websocket_requester.request(
            endpoint, request_message)
        result = result_getter()
        return self.handle_result(parse_response_fn, result)

    def authenticate_user(self, authenticate_request: AuthenticateRequest) -> AuthenticateResponse:
        request_data = Authenticator.convert_data(authenticate_request)
        return self.request(Endpoints.AUTHENTICATE_USER, EndpointCategory.AP, to_dict(request_data), lambda data: from_dict(AuthenticateResponse, data))

    def ping(self) -> None:
        result = self.request(
            Endpoints.PING, EndpointCategory.AP, None, parse_response_fn(Pong, ["msg"]))
        return

    def handle_result(self, parse_response_fn: Callable[[Any], T], result: Any) -> T:
        if result.is_left():
            raise result.get_left()
        data_dict = json.loads(result.get(), use_decimal=True)
        return ApDataHandler.handle_ap_data(parse_response_fn, data_dict)



from dataclasses import dataclass
from typing import List

from notbank_python_sdk.models.network_template import NetworkTemplate


@dataclass
class CurrencyNetworkTemplates:
    currency: str
    network: str
    network_name: str
    network_protocol: str
    template: List[NetworkTemplate]



import unittest

from notbank_python_sdk.notbank_client import NotbankClient
from tests import test_helper


class TestLogOut(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        connection = test_helper.new_rest_client_connection()
        cls.credentials = test_helper.load_credentials()
        test_helper.authenticate_connection(connection, cls.credentials)
        cls.client = NotbankClient(connection)

    def test_logout_success(self):
        """Prueba exitosa: cierre de sesión con un token válido."""
        self.client.logout()

    def test_logout_invalid_token(self):
        """Prueba: cierre de sesión con un token inválido."""
        self.client.logout()

    def test_logout_missing_token(self):
        """Prueba: cierre de sesión sin token."""
        self.client.logout()


if __name__ == "__main__":
    unittest.main()



from dataclasses import dataclass


@dataclass
class UserInfo:
    user_id: int
    user_name: str
    email: str
    password_hash: str
    pending_email_code: str
    email_verified: bool
    account_id: int
    date_time_created: str
    affiliate_id: int
    referer_id: int
    oms_id: int
    use_2fa: bool
    salt: str
    pending_code_time: str
    locked: bool
    locked_time: str
    number_of_failed_attempt: int
    margin_borrower_enabled: bool
    margin_acquisition_halt: bool
    operator_id: int



from dataclasses import dataclass
from decimal import Decimal


@dataclass
class InstrumentStatistic:
    oms_id: int
    account_id: int
    instrument_id: int
    instrument_symbol: str
    quantity_bought: Decimal
    quantity_sold: Decimal
    notional_bought: Decimal
    notional_sold: Decimal
    monthly_quantity_bought: Decimal
    monthly_quantity_sold: Decimal
    monthly_notional_bought: Decimal
    monthly_notional_sold: Decimal
    trade_volume: Decimal
    monthly_trade_volume: Decimal
    total_day_buys: int
    total_day_sells: int
    total_month_buys: int
    total_month_sells: int
    notional_conversion_rate: Decimal
    notional_conversion_symbol: str
    rolling_monthly_start_date: int
    last_trade_id: int
    daily_notional_trade_volume: Decimal
    monthly_notional_trade_volume: Decimal
    yearly_notional_trade_volume: Decimal



from dataclasses import dataclass
from typing import Optional

from notbank_python_sdk.requests_models.with_oms_id import WithOMSId


@dataclass
class OmsFeesRequest(WithOMSId):
    product_id: Optional[int] = None
    account_provider_id: Optional[int] = None



from dataclasses import dataclass
from decimal import Decimal

from notbank_python_sdk.requests_models.with_oms_id import WithOMSId


@dataclass
class ModifyOrderRequest(WithOMSId):
    order_id: int
    instrument_id: int
    quantity: Decimal
    account_id: int



from dataclasses import dataclass
from decimal import Decimal
from typing import List


@dataclass
class OrderBookLevel:
    quantity: Decimal
    price: Decimal


@dataclass
class OrderBook:
    timestamp: int
    bids: List[OrderBookLevel]
    asks: List[OrderBookLevel]


@dataclass
class OrderBookRaw:
    timestamp: int
    bids: List[List[Decimal]]
    asks: List[List[Decimal]]


def order_book_from_raw(raw: OrderBookRaw) -> OrderBook:
    bids = [
        OrderBookLevel(quantity=Decimal(level[0]), price=Decimal(level[1]))
        for level in raw.bids]
    asks = [
        OrderBookLevel(quantity=Decimal(level[0]), price=Decimal(level[1]))
        for level in raw.asks]
    return OrderBook(timestamp=raw.timestamp, bids=bids, asks=asks)



import simplejson as json
import unittest
from notbank_python_sdk.models.ticker import ticker_from_list


class ConvertTickerFromJsonTestCase(unittest.TestCase):
    def test_ticker_from_list(self):
        tickers_as_list_str = "[[1750743900000,104701.54,104701.54,104701.54,104701.54,0,0,0,154,1750743840000],[1750743960000,104701.54,104701.54,104701.54,104701.54,0,0,0,154,1750743900000],[1750744020000,104701.54,104701.54,104701.54,104701.54,0,0,0,154,1750743960000]]"
        tickers_as_list = json.loads(tickers_as_list_str, use_decimal=True)
        level2_tickers = [ticker_from_list(
            ticker_as_list) for ticker_as_list in tickers_as_list]



import unittest
from notbank_python_sdk.notbank_client import NotbankClient

from notbank_python_sdk.requests_models.get_order_status import GetOrderStatusRequest
from tests import test_helper


class TestGetOrderStatus(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        connection = test_helper.new_rest_client_connection()
        cls.credentials = test_helper.load_credentials()
        test_helper.authenticate_connection(connection, cls.credentials)
        cls.client = NotbankClient(connection)

    def test_get_order_status_success(self):
        """
        Prueba exitosa: obtiene el estado de una orden válida.
        """
        request = GetOrderStatusRequest(
            account_id=7,
            order_id=6562,
        )
        response = self.client.get_order_status(request)

        # Verificamos que la respuesta contiene los datos esperados
        self.assertIsNotNone(response)
        self.assertEqual(response.order_id, 6562)
        self.assertEqual(response.side, "Buy")
        self.assertEqual(response.price, 23436.0)
        self.assertEqual(response.account_name, "sample_user")


if __name__ == "__main__":
    unittest.main()



# models/get_enums.py
from typing import List
from dataclasses import dataclass, field


@dataclass
class EnumItem:
    name: str
    description: str
    number: int


@dataclass
class EnumsResponse:
    class_name: str
    property_name: str
    enums: List[EnumItem] = field(default_factory=list)

    @classmethod
    def from_dict(cls, d):
        return cls(
            class_name=d.get("Class"),
            property_name=d.get("Property"),
            enums=[EnumItem(**e) for e in d.get("Enums", [])]
        )



from dataclasses import dataclass
from decimal import Decimal
from uuid import UUID


@dataclass
class Quote:
    id: UUID
    is_inverse: bool
    type: int
    state: int
    currency_in: str
    currency_out: str
    amount_in: Decimal
    amount_out: Decimal
    amount_usdt_out: Decimal
    fee_amount: Decimal
    fee_detail: str



import os
import sys
import setuptools
from setuptools.command.test import test as TestCommand


CURRENT_PYTHON = sys.version_info[:2]
REQUIRED_PYTHON = (3, 7)

if CURRENT_PYTHON < REQUIRED_PYTHON:
    sys.stderr.write(
        """
        ==========================
        Unsupported Python version
        ==========================
        This version of Chunked String Serde requires at least Python {}.{}, but
        you're trying to install it on Python {}.{}. To resolve this,
        consider upgrading to a supported Python version.
        """.format(
            *(REQUIRED_PYTHON + CURRENT_PYTHON),
        )
    )
    sys.exit(1)


# "setup.py publish" shortcut.
if sys.argv[-1] == "publish":
    os.system('python setup.py sdist bdist_wheel')
    os.system('twine upload dist/*')
    sys.exit()


with open("README.md", "r", encoding="utf-8") as fh:
    long_description = fh.read()


with open("requirements.txt", "r", encoding="utf-8") as requeriments:
    REQUIREMENTS = requeriments.readlines()


class PyTest(TestCommand):
    user_options = [("pytest-args=", "a", "Arguments to pass into py.test")]

    def initialize_options(self):
        TestCommand.initialize_options(self)
        try:
            from multiprocessing import cpu_count

            self.pytest_args = ["-n", str(cpu_count()), "--boxed"]
        except (ImportError, NotImplementedError):
            self.pytest_args = ["-n", "1", "--boxed"]

    def finalize_options(self):
        TestCommand.finalize_options(self)
        self.test_args = []
        self.test_suite = True

    def run_tests(self):
        import pytest

        errno = pytest.main(self.pytest_args)
        sys.exit(errno)


test_requirements = [
    "pytest ==7.3.1",
]

setuptools.setup(
    name="notbank",
    version="2.0.3",
    packages=[
        "notbank_python_sdk",
        "notbank_python_sdk.core",
        "notbank_python_sdk.models",
        "notbank_python_sdk.requests_models",
        "notbank_python_sdk.rest",
        "notbank_python_sdk.websocket",
        "notbank_python_sdk.websocket.websocket_restarter",
    ],
    include_package_data=True,
    description="Notbank API client library",
    long_description=long_description,
    long_description_content_type="text/markdown",
    keywords=[
        "api", "notbank", "cryptomkt",
        "cryptomarket", "bitcoin", "client", "cryptocurrency"],
    url="https://github.com/notbank-exchange/notbank-python",
    author="Notbank",
    python_requires=">=3.7",
    install_requires=REQUIREMENTS,
    cmdclass={"test": PyTest},
    tests_require=test_requirements,
    classifiers=[
        "Intended Audience :: Developers",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3 :: Only",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Topic :: Software Development :: Libraries :: Python Modules",
    ],
)



from enum import Enum
from typing import Any


class Wrapper:
    wrapped_value: str

    def __init__(self, value: Any):
        self.wrapped_value = value


class EndpointCategory(Enum):
    AP = Wrapper('ap')
    NB = Wrapper('api/nb')
    NB_PAGE = Wrapper('api/nb')

    @property
    def val(self):
        return self.value.wrapped_value



import unittest
from notbank_python_sdk.requests_models.add_client_bank_account_request import AddClientBankAccountRequest, BankAccountKind

from tests import test_helper

from notbank_python_sdk.notbank_client import NotbankClient


class TestAddClientBankAccount(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        connection = test_helper.new_rest_client_connection()
        cls.credentials = test_helper.load_credentials()
        test_helper.authenticate_connection(connection, cls.credentials)
        cls.client = NotbankClient(connection)

    def test_add_client_bank_account(self):
        response = self.client.add_client_bank_account(AddClientBankAccountRequest(
            country="CL",
            bank="11",
            number="123123",
            kind=BankAccountKind.AHORRO,
        ))
        self.assertIsNotNone(response)


if __name__ == "__main__":
    unittest.main()



from dataclasses import dataclass
from typing import Optional


@dataclass
class GetUserReportWriterResultRecordsRequest:
    user_id: int
    depth: Optional[int] = None
    start_index: Optional[int] = None



import threading
import time
from typing import Callable
from concurrent.futures import Executor, Future, ThreadPoolExecutor, TimeoutError
from notbank_python_sdk.error import NotbankException
from notbank_python_sdk.websocket.synched_var import SynchedValue
from notbank_python_sdk.websocket.websocket_connection import WebsocketConnection


class Pinger:
    _executor: Executor
    _stop_requested: SynchedValue[bool]
    _ping_interval_seconds: int
    _ping_timeout: int

    def __init__(
        self,
        executor: Executor,
        stop_requested: SynchedValue[bool],
        ping_interval_seconds: int,
        ping_timeout: int
    ) -> None:
        self._executor = executor
        self._stop_requested = stop_requested
        self._ping_interval_seconds = ping_interval_seconds
        self._ping_timeout = ping_timeout

    @staticmethod
    def create() -> 'Pinger':
        return Pinger(
            executor=ThreadPoolExecutor(1, "notbank pinger pool"),
            stop_requested=SynchedValue.create(False),
            ping_interval_seconds=5,
            ping_timeout=5
        )

    def _reset(self) -> None:
        self._stop_requested.set(False)

    def _should_stop(self) -> bool:
        return self._stop_requested.get()

    def _ping(self, ping_fn: Callable[[], None]) -> Future:
        return self._executor.submit(ping_fn)

    def _ping_forever(self, ping_fn: Callable[[], None], reconnect_fn: Callable[[], None]):
        while True:
            time.sleep(self._ping_interval_seconds)
            if self._should_stop():
                return
            try:
                future_pong = self._ping(ping_fn)
                future_pong.result(self._ping_timeout)
            except (TimeoutError, NotbankException) as e:
                if not self._should_stop():
                    reconnect_fn()
                return

    def restart(self, ping_fn: Callable[[], None], reconnect_fn: Callable[[], None]):
        self._stop_requested.set(False)
        threading.Thread(
            target=self._ping_forever,
            args=[ping_fn, reconnect_fn],
            name="notbank pinger",
            daemon=True
        ).start()

    def stop(self) -> None:
        self._stop_requested.set(True)



import unittest

from notbank_python_sdk.notbank_client import NotbankClient

from notbank_python_sdk.requests_models.get_instruments_request import GetInstrumentsRequest
from tests import test_helper


class TestGetInstruments(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        connection = test_helper.new_rest_client_connection()
        cls.credentials = test_helper.load_credentials()
        test_helper.authenticate_connection(connection, cls.credentials)
        cls.client = NotbankClient(connection)

    def test_get_instruments_success(self):
        request = GetInstrumentsRequest(instrument_state="Both")
        response = self.client.get_instruments(request)

        self.assertIsNotNone(response)
        self.assertIsInstance(response, list)
        self.assertEqual(len(response), 95)
        self.assertEqual(response[0].symbol, "USDTDAI")
        self.assertEqual(response[0].product1_symbol, "USDT")
        self.assertEqual(response[1].symbol, "XRPBTC")
        self.assertEqual(response[1].product1_symbol, "XRP")

    def test_get_instruments_failure(self):
        request = GetInstrumentsRequest()
        response = self.client.get_instruments(request)
        self.assertIsNotNone(response)


if __name__ == "__main__":
    unittest.main()



from dataclasses import dataclass
from typing import Union


@dataclass
class AuthenticateRequest:
    api_public_key: str
    api_secret_key: str
    user_id: Union[str, int]


@dataclass
class AuthenticateRequestData:
    api_key: str
    signature: str
    user_id: str
    nonce: str



import unittest
from notbank_python_sdk.requests_models.deposit_address_request import DepositAddressRequest

from tests import test_helper

from notbank_python_sdk.notbank_client import NotbankClient


class TestCreateDepositAddress(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        connection = test_helper.new_rest_client_connection()
        cls.credentials = test_helper.load_credentials()        
        test_helper.authenticate_connection(connection, cls.credentials)
        cls.client = NotbankClient(connection)

    def test_create_deposit_address(self):
        response = self.client.create_deposit_address(DepositAddressRequest(self.credentials.account_id, "BTC","BTC_TEST"))
        self.assertIsNotNone(response)


if __name__ == "__main__":
    unittest.main()



import unittest
from notbank_python_sdk.notbank_client import NotbankClient

from notbank_python_sdk.requests_models.trades import TradesRequest
from tests import test_helper


class TestTrades(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        connection = test_helper.new_rest_client_connection()
        cls.credentials = test_helper.load_credentials()
        test_helper.authenticate_connection(connection, cls.credentials)
        cls.client = NotbankClient(connection)

    def test_get_trades_success(self):
        request = TradesRequest(
            market_pair="BTCUSD"
        )
        response = self.client.get_trades(request)
        self.assertIsInstance(response, list)
        self.assertEqual(len(response), 2)
        self.assertEqual(response[0].trade_id, 1)
        self.assertEqual(response[0].price, 29000)
        self.assertEqual(response[0].type, "buy")
        self.assertEqual(response[1].trade_id, 2)
        self.assertEqual(response[1].price, 28800)
        self.assertEqual(response[1].type, "sell")


if __name__ == "__main__":
    unittest.main()



from dataclasses import dataclass
from queue import Empty, Queue
from threading import Thread
from typing import Any, Callable, Optional
from notbank_python_sdk.websocket.synched_var import SynchedValue

import websocket

from notbank_python_sdk.error import ErrorCode, NotbankException, NotbankException


@dataclass
class Hooks:
    on_message: Callable[[Any, str], None]
    on_open: Callable[[Any], None]
    on_close: Callable[[Any, int, str], None]
    on_error: Callable[[Any, Exception], None]


class WebsocketManager:
    _uri: str
    _connected_signal: Queue
    _peek_message_out: Callable[[str], None]
    _connected: SynchedValue[bool]
    _hooks: Hooks
    _ws: websocket.WebSocketApp
    _thread: Optional[Thread]

    def __init__(self,
                 handler,
                 host,
                 peek_message_in: Callable[[str], None] = lambda x: None,
                 peek_message_out: Callable[[str], None] = lambda x: None):
        self._uri = self._build_url(host)
        self._connected = SynchedValue.create(False)
        self._connected_signal = Queue(1)
        self._peek_message_out = peek_message_out
        self._thread = None

        def on_message(ws, message):
            peek_message_in(message)
            handler.handle(message)

        def on_error(ws, error):
            handler.on_error(error)

        def on_close(ws, code: int, message: str):
            handler.on_close(code, message)

        def on_open(ws):
            self._connected.set(True)
            self._connected_signal.put(True)
            handler.on_open()
        self._hooks = Hooks(
            on_message=on_message,
            on_open=on_open,
            on_close=on_close,
            on_error=on_error
        )

    def connect(self, timeout: float = 5):
        self._connected.set(False)
        self._ws = websocket.WebSocketApp(
            self._uri,
            on_message=self._hooks.on_message,
            on_error=self._hooks.on_error,
            on_close=self._hooks.on_close,
            on_open=self._hooks.on_open,
        )
        self._thread = Thread(
            target=self._ws.run_forever,
            name="notbank websocket",
            daemon=True)
        self._thread.start()
        try:
            self._connected_signal.get(block=True, timeout=timeout)
            return
        except Empty:
            self.close()
            raise NotbankException(
                ErrorCode.TIMED_OUT, "unable to connect to server. connection timed out")

    def send(self, msg: str) -> None:
        if not self._connected.get() or self._thread is None or not self._thread.is_alive():
            raise NotbankException(
                ErrorCode.OPERATION_FAILED, "websocket not connected")
        self._peek_message_out(msg)
        self._ws.send(msg)

    def close(self):
        try:
            self._ws.close()
            self._connected.set(False)
        except Exception as e:
            self._hooks.on_error(self._ws, e)
        if self._thread is None:
            return
        self._thread.join(5)

    def _build_url(self, host: str) -> str:
        return "wss://" + host + "/wsgateway/"



from typing import Callable, Dict, Optional


class SubscriptionCallbacks:
    _callbacks: Dict[str,  Callable[[str], None]]

    def __init__(self, callbacks: Dict[str,  Callable[[str], None]]):
        self._callbacks = callbacks

    @staticmethod
    def create():
        return SubscriptionCallbacks({})

    def add(self,  callback_id: str, handler: Callable[[str], None]) -> None:
        self._callbacks[callback_id] = handler

    def get(self, callback_id: str) -> Optional[Callable[[str], None]]:
        return self._callbacks.get(callback_id)

    def remove(self, callback_id: str) -> None:
        if callback_id in self._callbacks:
            del self._callbacks[callback_id]

    def close(self) -> None:
        self._callbacks.clear()



from dataclasses import dataclass
from typing import Optional


@dataclass
class WebAuthenticateUser:
    authenticated: bool
    session_token: Optional[str] = None
    user_id: Optional[str] = None
    two_fa_token: Optional[str] = None
    requires_2fa: Optional[bool] = None
    auth_type: Optional[str] = None
    errormsg: Optional[str] = None





from dataclasses import dataclass


@dataclass
class CancelOrderRejectEvent:
    oms_id: int
    account_id: int
    order_id: int
    order_revision: int
    order_type: str
    instrument_id: int
    status: str
    reject_reason: str



from dataclasses import dataclass
from decimal import Decimal
from typing import Optional

from notbank_python_sdk.requests_models.with_oms_id import WithOMSId


@dataclass
class GetDepositFeeRequest(WithOMSId):
    account_id: int
    product_id: int
    amount: Decimal
    account_provider_id: Optional[int] = None



import unittest

from notbank_python_sdk.notbank_client import NotbankClient

from notbank_python_sdk.requests_models.verification_level_config_request import VerificationLevelConfigRequest
from tests import test_helper


class TestGetInstrumentVerificationLevelConfig(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        connection = test_helper.new_rest_client_connection()
        cls.credentials = test_helper.load_credentials()
        test_helper.authenticate_connection(connection, cls.credentials)
        cls.client = NotbankClient(connection)

    def test_get_instrument_verification_level_config_success(self):
        """
        Prueba exitosa: Solicitud válida, devuelve la configuración de niveles de verificación.
        """
        request = VerificationLevelConfigRequest(
            account_id=7,
        )
        response = self.client.get_instrument_verification_level_config(
            request)

        # Verificaciones
        self.assertIsNotNone(response)
        self.assertEqual(response.level, 1)
        self.assertEqual(len(response.instruments), 2)
        self.assertEqual(response.instruments[0].instrument_name, "BTCUSD")
        self.assertEqual(response.instruments[1].instrument_name, "ETHUSD")

    def test_get_instrument_verification_level_config_not_found(self):
        """
        Prueba: Solicitud inválida, no se encuentra la configuración.
        """
        request = VerificationLevelConfigRequest(
            account_id=999,  # account_id inválido
        )
        response = self.client.get_instrument_verification_level_config(
            request)

        # Verificaciones
        self.assertIsNone(response)


if __name__ == "__main__":
    unittest.main()





from concurrent.futures import Executor, ThreadPoolExecutor, TimeoutError

from typing import Any, Optional, TypeVar
from notbank_python_sdk.error import ErrorCode, NotbankException
from notbank_python_sdk.websocket.callback_manager import CallbackManager
from notbank_python_sdk.websocket.connection_configuration import ConnectionConfiguration
from notbank_python_sdk.websocket.handler import WebsocketHandler
from notbank_python_sdk.websocket.websocket_restarter.pinger import Pinger
from notbank_python_sdk.websocket.websocket_restarter.reauther import Reauther
from notbank_python_sdk.websocket.websocket_restarter.resubscriber import Resubscriber
from notbank_python_sdk.websocket.synched_var import SynchedValue
from notbank_python_sdk.websocket.websocket_connection import WebsocketConnection
from notbank_python_sdk.websocket.websocket_manager import WebsocketManager
from notbank_python_sdk.websocket.websocket_requester import WebsocketRequester
from notbank_python_sdk.websocket.websocket_response_handler import WebsocketResponseHandler

T = TypeVar('T')

class Restarter:
    _executor: Executor
    _connection: Optional[WebsocketConnection]
    _connection_configuration: ConnectionConfiguration
    _pinger: Pinger
    _resubscriber: Resubscriber
    _reauther: Reauther
    _close_requested: SynchedValue[bool]
    _reconnecting: SynchedValue[bool]

    def __init__(
        self,
        executor: Executor,
        connection: Optional[WebsocketConnection],
        connection_configuration: ConnectionConfiguration,
        pinger: Pinger,
        resubscriber: Resubscriber,
        reauther: Reauther,
        requested_close: SynchedValue[bool],
        reconnecting: SynchedValue[bool],
    ):
        self._executor = executor
        self._connection = connection
        self._connection_configuration = connection_configuration
        self._pinger = pinger
        self._resubscriber = resubscriber
        self._reauther = reauther
        self._close_requested = requested_close
        self._reconnecting = reconnecting

    @staticmethod
    def create(connection_configuration: ConnectionConfiguration) -> 'Restarter':
        executor = ThreadPoolExecutor(1, "notbank restarter pool")
        return Restarter(
            executor,
            connection=None,
            connection_configuration=connection_configuration,
            pinger=Pinger(executor, SynchedValue.create(False), 10, 5),
            resubscriber=Resubscriber(),
            reauther=Reauther(),
            requested_close=SynchedValue.create(False),
            reconnecting=SynchedValue.create(False)
        )

    def get_resubscriber(self) -> Resubscriber:
        return self._resubscriber

    def get_reauther(self) -> Reauther:
        return self._reauther

    def get_connection(self) -> WebsocketConnection:
        if self._connection is None:
            raise NotbankException(
                ErrorCode.OPERATION_FAILED, "websocket not started")
        if self._reconnecting.get():
            raise NotbankException(
                ErrorCode.OPERATION_FAILED, "websocket reconnecting")
        return self._connection

    def _on_close(self, code: Any, message: str) -> None:
        if self._reconnecting or not self._close_requested.get():
            return
        self._connection_configuration.on_close(code, message)

    def _on_failure(self, exception: Exception) -> None:
        if self._reconnecting:
            return
        self._connection_configuration.on_failure(exception)

    def _new_connection(self) -> WebsocketConnection:
        callback_manager = CallbackManager.create()
        websocket_response_handler = WebsocketResponseHandler.create(
            callback_manager,
            self._connection_configuration.on_failure)
        websocket_handler = WebsocketHandler(
            websocket_response_handler.handle,
            on_open=self._connection_configuration.on_open,
            on_close=self._on_close,
            on_failure=self._on_failure)
        websocket_manager = WebsocketManager(
            websocket_handler,
            self._connection_configuration.uri,
            self._connection_configuration.peek_message_in,
            self._connection_configuration.peek_message_out)
        websocket_requester = WebsocketRequester.create(
            callback_manager,
            websocket_manager.send,
            self._on_failure,
            request_timeout=self._connection_configuration.request_timeout,
        )
        websocket_response_handler = WebsocketResponseHandler.create(
            callback_manager,
            self._on_failure)
        return WebsocketConnection(
            callback_manager,
            websocket_manager,
            websocket_requester,
            websocket_response_handler)

    def _close_current_connection(self) -> None:
        self._pinger.stop()
        if self._connection is not None:
            self._connection.close()

    def _connect(self) -> None:
        if self._connection is None:
            raise NotbankException(
                ErrorCode.OPERATION_FAILED, "websocket not started")
        while not self._close_requested.get():
            connected_future = self._executor.submit(self._connection.connect)
            try:
                connected_future.result(10)
                return
            except (TimeoutError, NotbankException):
                # try again
                pass

    def close(self) -> None:
        self._close_requested.set(True)
        self._close_current_connection()

    def reconnect(self) -> None:
        if self._reconnecting.get() or self._close_requested.get():
            return
        self._reconnecting.set(True)
        self._close_current_connection()
        self._connection = self._new_connection()
        self._connect()
        self._reauther.reauthenticate(self._connection)
        self._resubscriber.resubscribe(self._connection)
        self._pinger.restart(self._connection.ping, self.reconnect)
        self._reconnecting.set(False)



from decimal import Decimal
from unittest import TestCase, main

from notbank_python_sdk.models.oms_fee import OmsFee
from notbank_python_sdk.requests_models.account_fees_request import AccountFeesRequest
from notbank_python_sdk.core.converter import from_dict, from_json_str, to_dict
from notbank_python_sdk.websocket.message_frame import MessageFrame


class ModelConversionTestCase(TestCase):
    def test_oms_id_to_dict(self):
        account_fee_request = AccountFeesRequest(account_id=9)
        data = to_dict(account_fee_request)
        self.assertEqual(data["OMSId"], account_fee_request.oms_id)
        self.assertEqual(data["AccountId"], account_fee_request.account_id)

    def test_from_dict(self):
        data = {
            "OMSId": 111,
            "AccountId": 222,
            "AccountProviderId": 333,
            "FeeId": 444,
            "FeeAmt": 5.55,
            "FeeCalcType": "aType",
            "IsActive": True,
            "ProductId": 666,
            "MinimalFeeAmt": 7.77,
            "MinimalFeeCalcType": "anotherType"
        }
        deposit_fee = from_dict(OmsFee, data)

        self.assertEqual(deposit_fee.fee_amt, Decimal('5.55'))
        self.assertEqual(deposit_fee.oms_id, 111)

    def test_to_dict(self):
        deposit_fee = OmsFee(
            oms_id=111,
            account_id=222,
            account_provider_id=333,
            fee_id=444,
            fee_amt=Decimal('5.55'),
            fee_calc_type='aType',
            is_active=True,
            product_id=666,
            minimal_fee_amt=Decimal('7.77'),
            minimal_fee_calc_type="anotherType"
        )
        data = to_dict(deposit_fee)
        self.assertEqual(data['OMSId'], deposit_fee.oms_id)
        self.assertEqual(data['FeeAmt'], str(deposit_fee.fee_amt))

    def test_json_str_to_message_frame(self):
        json_str = '{"m":1, "i":2, "n":"SubscribeLevel1", "o":"{ \\"OMSId\\":1 }"}'
        message_frame = from_json_str(MessageFrame, json_str)
        self.assertTrue(message_frame.is_right(),
                        lambda: message_frame.get_left())


if __name__ == '__main__':
    main()



from dataclasses import dataclass
from decimal import Decimal


@dataclass
class TickerSummary:
    base_id: int
    quote_id: int
    last_price: Decimal
    base_volume: Decimal
    quote_volume: Decimal



from typing import List
import unittest
from notbank_python_sdk.notbank_client import NotbankClient

from notbank_python_sdk.models.level2_ticker_snapshot import Level2TickerSnapshot
from notbank_python_sdk.requests_models.get_l2_snapshot import GetL2SnapshotRequest
from tests import test_helper


class TestGetL2Snapshot(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        connection = test_helper.new_rest_client_connection()
        cls.credentials = test_helper.load_credentials()
        test_helper.authenticate_connection(connection, cls.credentials)
        cls.client = NotbankClient(connection)

    def test_get_l2_snapshot_success(self):
        request = GetL2SnapshotRequest(
            instrument_id=1,
            depth=10
        )
        response = self.client.get_l2_snapshot(request)
        self.assertIsInstance(response, List[Level2TickerSnapshot])
        self.assertEqual(len(response), 2)
        self.assertEqual(response[0].md_update_id, 26)
        self.assertEqual(response[0].price, 28700)
        self.assertEqual(response[0].side, 0)


if __name__ == "__main__":
    unittest.main()



from typing import Optional
from dataclasses import dataclass

from notbank_python_sdk.requests_models.with_oms_id import WithOMSId


@dataclass
class CancelAllOrdersRequest(WithOMSId):
    account_id: Optional[int] = None
    instrument_id: Optional[int] = None



from decimal import Decimal
import unittest

from notbank_python_sdk.notbank_client import NotbankClient

from notbank_python_sdk.requests_models.fee_request import FeeRequest
from tests import test_helper


class TestGetDepositFee(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        connection = test_helper.new_rest_client_connection()
        cls.credentials = test_helper.load_credentials()
        test_helper.authenticate_connection(connection, cls.credentials)
        cls.client = NotbankClient(connection)

    def test_get_deposit_fee_success(self):
        """
        Prueba exitosa: solicitud válida, devuelve el fee estimado.
        """
        request = FeeRequest(
            account_id=1,
            product_id=1,
            amount=Decimal(100),
            account_provider_id=5,  # Opcional
        )
        response = self.client.get_deposit_fee(request)

        # Verificaciones
        self.assertIsNotNone(response)
        self.assertEqual(response.fee_amount, 1.0)
        self.assertEqual(response.ticket_amount, 100)

    def test_get_deposit_fee_default(self):
        """
        Prueba: solicitud con valores predeterminados, devuelve el fee estimado.
        """
        request = FeeRequest(
            account_id=1,
            product_id=1,
            amount=Decimal(50),
            account_provider_id=0,  # Valor predeterminado
        )
        response = self.client.get_deposit_fee(request)

        # Verificaciones
        self.assertIsNotNone(response)
        self.assertEqual(response.fee_amount, 0.5)
        self.assertEqual(response.ticket_amount, 50)

    def test_get_deposit_fee_not_found(self):
        request = FeeRequest(
            account_id=999,  # account_id inválido
            product_id=1,
            amount=Decimal(100),
            account_provider_id=5,
        )
        response = self.client.get_deposit_fee(request)

        # Verificaciones
        self.assertIsNone(response)


if __name__ == "__main__":
    unittest.main()



import unittest

from notbank_python_sdk.requests_models.cancel_order_request import CancelOrderRequest
from notbank_python_sdk.notbank_client import NotbankClient
from tests import test_helper


class TestCancelOrder(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        connection = test_helper.new_rest_client_connection()
        cls.credentials = test_helper.load_credentials()
        test_helper.authenticate_connection(connection, cls.credentials)
        cls.client = NotbankClient(connection)

    def test_cancel_order_success_order_id(self):
        request = CancelOrderRequest(
            account_id=self.credentials.account_id,
            order_id=6500
        )
        try:
            response = self.client.cancel_order(request)
        except Exception as e:
            self.fail(f"Cancel order failed with exception: {e}")

    def test_cancel_order_success_cl_order_id(self):
        request = CancelOrderRequest(
            account_id=self.credentials.account_id,
            cl_order_id=777
        )
        try:
            response = self.client.cancel_order(request)
        except Exception as e:
            self.fail(f"Cancel order failed with exception: {e}")

    def test_cancel_order_not_authorized(self):
        request = CancelOrderRequest(
            account_id=self.credentials.account_id+123,
            cl_order_id=9999
        )
        try:
            self.client.cancel_order(request)
            self.fail("Expected NotbankException but none was raised")
        except Exception as e:
            pass


if __name__ == "__main__":
    unittest.main()




import unittest
from notbank_python_sdk.notbank_client import NotbankClient

from notbank_python_sdk.models.user_info import UserInfo
from notbank_python_sdk.requests_models.get_user_info import GetUserInfoRequest
from tests import test_helper


class TestGetUserInfo(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        connection = test_helper.new_rest_client_connection()
        cls.credentials = test_helper.load_credentials()
        test_helper.authenticate_connection(connection, cls.credentials)
        cls.client = NotbankClient(connection)

    def test_get_user_info_response_ok(self):
        """
        Prueba exitosa: obtiene la información de usuario correcta según el mock.
        """
        req = GetUserInfoRequest(user_id=1)
        response = self.client.get_user_info(req)

        self.assertIsInstance(response, UserInfo)
        self.assertEqual(response.user_id, 1)
        self.assertEqual(response.user_name, "usuario_test")
        self.assertEqual(response.email, "test@email.com")
        self.assertEqual(response.account_id, 10)
        self.assertTrue(response.email_verified)
        self.assertFalse(response.locked)
        self.assertTrue(response.margin_borrower_enabled)


if __name__ == "__main__":
    unittest.main()



import unittest
from notbank_python_sdk.notbank_client import NotbankClient

from notbank_python_sdk.requests_models.user_devices import GetUserDevicesRequest
from tests import test_helper


class TestGetUserDevices(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        connection = test_helper.new_rest_client_connection()
        cls.credentials = test_helper.load_credentials()
        test_helper.authenticate_connection(connection, cls.credentials)
        cls.client = NotbankClient(connection)

    def test_get_user_devices_success(self):
        request = GetUserDevicesRequest(user_id=6)
        response = self.client.get_user_devices(request)
        self.assertIsInstance(response, list)
        self.assertEqual(len(response), 2)
        self.assertEqual(response[0].hash_code, 345283502)
        self.assertEqual(response[1].is_trusted, True)

    def test_get_user_devices_no_results(self):
        request = GetUserDevicesRequest(user_id=100)
        response = self.client.get_user_devices(request)
        self.assertIsInstance(response, list)
        self.assertEqual(response, [])

    def test_get_user_devices_no_param(self):
        # Simula la consulta de dispositivos para el usuario autenticado
        request = GetUserDevicesRequest()
        response = self.client.get_user_devices(request)
        self.assertIsInstance(response, list)
        self.assertGreaterEqual(len(response), 1)
        self.assertEqual(response[0].device_name, "Mac OS X 10.15")
        self.assertTrue(response[0].is_trusted)


if __name__ == "__main__":
    unittest.main()



from decimal import Decimal
import unittest
from notbank_python_sdk.notbank_client import NotbankClient

from notbank_python_sdk.requests_models.modify_order import ModifyOrderRequest
from notbank_python_sdk.core.endpoints import Endpoints
from tests import test_helper


class TestModifyOrder(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        connection = test_helper.new_rest_client_connection()
        cls.credentials = test_helper.load_credentials()
        test_helper.authenticate_connection(connection, cls.credentials)
        cls.client = NotbankClient(connection)

    def test_modify_order_success(self):
        """Prueba exitosa: modificación de orden válida."""
        request = ModifyOrderRequest(
            order_id=6507,
            instrument_id=9,
            quantity=Decimal(0.1),
            account_id=9,
        )
        self.client.modify_order(request)

    def test_modify_order_invalid_quantity(self):
        """Prueba fallida: cantidad inválida (mayor que la cantidad actual)."""
        request = ModifyOrderRequest(
            order_id=6507,
            instrument_id=9,
            quantity=Decimal(1.5),  # Cantidad inválida
            account_id=9,
        )
        self.client.modify_order(request)

if __name__ == "__main__":
    unittest.main()



import unittest
from uuid import UUID
from notbank_python_sdk.requests_models.confirm_whitelisted_address_request import ConfirmWhiteListedAddressRequest

from tests import test_helper

from notbank_python_sdk.notbank_client import NotbankClient


class TestCreateDepositAddress(unittest.TestCase):

    @classmethod
    def setUpClass(cls):
        connection = test_helper.new_rest_client_connection(
            test_helper.print_message_in, test_helper.print_message_out)
        cls.credentials = test_helper.load_credentials()
        test_helper.authenticate_connection(connection, cls.credentials)
        cls.client = NotbankClient(connection)

    def test_create_deposit_address(self):
        response = self.client.confirm_whitelisted_address(ConfirmWhiteListedAddressRequest(
            whitelisted_address_id=UUID(
                "ab38462b-e6ec-435c-a67c-e4c3e29f5b72"),
            account_id=self.credentials.account_id,
            sms_code="0564225"))
        self.assertIsNone(response)


if __name__ == "__main__":
    unittest.main()



from dataclasses import dataclass
from enum import Enum


class Action(str, Enum):
    ENABLE = "enable"
    DISABLE = "disable"


@dataclass
class UpdateOneStepWithdrawRequest:
    account_id: int
    action: Action
    otp: str



import unittest
from notbank_python_sdk.notbank_client import NotbankClient

from notbank_python_sdk.requests_models.get_instrument_request import GetInstrumentRequest
from tests import test_helper


class TestGetInstrument(unittest.TestCase):
    @classmethod
    def setUpClass(cls):
        connection = test_helper.new_rest_client_connection()
        cls.credentials = test_helper.load_credentials()
        test_helper.authenticate_connection(connection, cls.credentials)
        cls.client = NotbankClient(connection)

    def test_get_instrument_success(self):
        """
        Prueba exitosa: oms_id e instrument_id válidos, devuelve los detalles del instrumento.
        """
        request = GetInstrumentRequest(
            instrument_id=1,
        )
        response = self.client.get_instrument(request)

        # Verificaciones
        self.assertIsNotNone(response)
        self.assertEqual(response.symbol, "BTCUSD")
        self.assertEqual(response.product1_symbol, "BTC")
        self.assertEqual(response.session_status, "Running")
        self.assertEqual(response.minimum_quantity, 0.001)
        self.assertEqual(response.minimum_price, 10.0)

    def test_get_instrument_not_found(self):
        """
        Prueba: Instrumento no encontrado, devuelve None.
        """
        request = GetInstrumentRequest(
            instrument_id=999,  # instrument_id inválido
        )
        response = self.client.get_instrument(request)

        # Verificaciones
        self.assertIsNone(response)


if __name__ == "__main__":
    unittest.main()
